/**@file

This library contains functions associated with the JCOP Framework System Overview tool. The 
functions read the configuration file and build a hierarchy of systems and projects within 
each system. Then the Projects are polled simultaneously via the PMON port and the managers
information and states are updated in datapoint

@par Creation Date
	15/02/2007

@par Modification History
	
@par Constraints
	fwSystemOverview_readConfigFile has to be used to create the data point structure, 
        upon which rest of the functions act

@author 
	Kuldeep Joshi (BARC, India) mailto:kuldeep.joshi@gmail.com
*/

//@{

// Internal Variables
bool showErrorDialog = true;
mixed fwSystemOverviewFunc_mutex;

 
//Datapoint Types
const string fwSystemOverviewFunc_SYSCOLL_TYPE      = "_FwSystemOverviewList";                                  
const string fwSystemOverviewFunc_SYS_NODE_TYPE     = "_FwSystemOverviewSystem";
const string fwSystemOverviewFunc_PROJ_NODE_TYPE    = "_FwSystemOverviewProject";

//Datapoints prefix
const string fwSystemOverviewFunc_PROJVIEW_NODE     = "fw_SysView_Project_";
const string fwSystemOverviewFunc_SYSVIEW_NODE      = "fw_SysView_System_";
const string fwSystemOverviewFunc_DPNAME_ARRAY      = "fw_SysView_Collection";
                                  
//Datapoint Elements for fwSystemOverviewFunc_SYSCOLL_TYPE
const string fwSystemOverviewFunc_SYSLIST_STR       = ".SysList";
const string fwSystemOverviewFunc_CONF_FILE_STR     = ".ConfigFilePath";
const string fwSystemOverviewFunc_REFRESH_RATE_STR  = ".RefreshRate";

//Common Datapoint Elements
const string fwSystemOverviewFunc_CURR_STATE_STR    = ".CurrentState";
const string fwSystemOverviewFunc_OK_STATE_STR      = ".OkState";

//Datapoint Elements for fwSystemOverviewFunc_SYS_NODE_TYPE
const string fwSystemOverviewFunc_SYSNAME_STR       = ".SysName";
const string fwSystemOverviewFunc_SYSNO_STR         = ".SysNo";
const string fwSystemOverviewFunc_PROJLIST_STR      = ".ProjList";
const string fwSystemOverviewFunc_CHILD_STR         = ".ChildSysNoList";
const string fwSystemOverviewFunc_PARENT_STR        = ".ParentSysNo";


//Datapoint Elements for fwSystemOverviewFunc_PROJ_NODE_TYPE
const string fwSystemOverviewFunc_DM_PORT_STR       = ".DmPort";
const string fwSystemOverviewFunc_DISTMAN_PORT_STR  = ".DistPort";
const string fwSystemOverviewFunc_EVM_PORT_STR      = ".EVMPort";
const string fwSystemOverviewFunc_IP_STR            = ".HostIP";
const string fwSystemOverviewFunc_HOSTNAME_STR      = ".HostName";
const string fwSystemOverviewFunc_PMON_PORT_STR     = ".PmonPort";
const string fwSystemOverviewFunc_PROJ_PATH_STR     = ".SharedDirPath";
const string fwSystemOverviewFunc_PROJ_NAME_STR     = ".ExpectedProjectName";
const string fwSystemOverviewFunc_ACT_PROJ_NAME_STR = ".ActualProjectName";

const string fwSystemOverviewFunc_USER_NAME_STR     = ".UserName";
const string fwSystemOverviewFunc_USER_PASSWORD_STR = ".UserPassword";

const string fwSystemOverviewFunc_POLL_PROJ_STR     = ".PollProject";
const string fwSystemOverviewFunc_IS_POLL_PROJ_STR  = ".IsPollingProject";

// Manager String Paths
const string fwSystemOverviewFunc_MANAGER_STR       = ".Managers";

// Manager Elements
const string fwSystemOverviewFunc_INDEX_STR         = ".Index";
const string fwSystemOverviewFunc_MODE_STR          = ".Mode";
const string fwSystemOverviewFunc_OPTIONS_STR       = ".Options";
const string fwSystemOverviewFunc_PID_STR           = ".PID";
const string fwSystemOverviewFunc_RESETMIN_STR      = ".ResetMin";
const string fwSystemOverviewFunc_RESTART_STR       = ".Restart#";
const string fwSystemOverviewFunc_SECKILL_STR       = ".SecKill";
const string fwSystemOverviewFunc_STARTTIME_STR     = ".StartTime";
const string fwSystemOverviewFunc_TYPE_STR          = ".Type";
const string fwSystemOverviewFunc_NUM_STR           = ".Num";
const string fwSystemOverviewFunc_UPDATED_STR       = ".LastUpdated";


// States Constants
const int fwSystemOverviewFunc_STOPPED_NORMAL    = 0;
const int fwSystemOverviewFunc_INITIALIZE        = 1;
const int fwSystemOverviewFunc_RUNNING           = 2;
const int fwSystemOverviewFunc_BLOCKED           = 3;
const int fwSystemOverviewFunc_STOPPED_ABNORMAL  = 10;
const int fwSystemOverviewFunc_RAPID_RESTART     = 11;
const int fwSystemOverviewFunc_PROJNAME_MISMATCH = 12;
const int fwSystemOverviewFunc_PMON_NO_RESPONSE  = 13;
const int fwSystemOverviewFunc_PMON_NOT_RUNNING  = 14;

//@}

//@{





/** Extracts the the project parameters, manager and ok state information for the project.
  
    NOTE: This is one of the functions to read configuration information, save data in 
    datapoint and initialise the data structure required by fwSystemOverview tool

@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param projStr  String which contained project parameters is parsed and the projDp datapoint is populated

@param sysNo    The number of the system for which the project is configured

@param projDp   Datapoint name where the parsed project information is stored
*/

bool flagGetHost = true;
bool flagGetIP  = true;

/** Converts state in integer format to a string format. 
    NOTE: By modifying this function one can add additional states
@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param stat   Integer state
*/

string fwSystemOverviewFunc_getStrFromState(int stat)
{
  string str;
  
  switch(stat){
    case fwSystemOverviewFunc_STOPPED_NORMAL:
      str = "STOPPED";
      break;
    case fwSystemOverviewFunc_INITIALIZE:
      str = "INITIALIZE";
      break;
    case fwSystemOverviewFunc_RUNNING:
      str = "RUNNING";
      break;
    case fwSystemOverviewFunc_BLOCKED:
      str = "BLOCKED";
      break;
    case fwSystemOverviewFunc_STOPPED_ABNORMAL:
      str = "STOPPED ABNORMAL";
      break;
    case fwSystemOverviewFunc_RAPID_RESTART:
      str = "RAPID RESTART";
      break;
    case fwSystemOverviewFunc_PROJNAME_MISMATCH:
      str = "PROJECT NAME MISMATCH";
      break;
    case fwSystemOverviewFunc_PMON_NO_RESPONSE:
      str = "PMON NOT RESPONDING";
      break;
    case fwSystemOverviewFunc_PMON_NOT_RUNNING:
      str = "PMON NOT RUNNING";
      break;
    default:
      str = "STATE NOT DEFINED";
      break;
   }
  
  return str;
}


/** Sets the projects current state as PMON_NOT_RUNNING
@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param projDp  Project datapoint name
*/

int fwSystemOverviewFunc_setProjPmonNotRunning(string projDp)
{
  return dpSet(projDp + fwSystemOverviewFunc_CURR_STATE_STR, fwSystemOverviewFunc_PMON_NOT_RUNNING);
}

/** Sets the projects current state as PMON_NOT_RESPONDING
@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param projDp  Project datapoint name
*/
int fwSystemOverviewFunc_setProjPmonNotResponding(string projDp)
{
  return dpSet(projDp + fwSystemOverviewFunc_CURR_STATE_STR, fwSystemOverviewFunc_PMON_NO_RESPONSE);
}



/** Gets all the information about the managers in project. 
    Note: The relation between data from all the arrays is based on the Index of the array.
@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param projDp        Project datapoint name
@param type          Array of manager types
@param num           Array of manager number
@param opt           Array of manager options
@param mode          Array of manager startup mode
@param index         Array of managet index
@param cStat         Array of current state of managers
@param okStat        Array of Ok States of manager as defined in the configuration file. This array can be shorter than the other arrays
@param pid           Array of process ids of manager
@param resetMin      Array of reset start counter for manager
@param restart       Array of restart for manager
@param secKill       Array of Seconds to Kill for manager
@param startTime     Array of start time for manager
@param lastUpdated   Array of flags that indicated if the manager was updated during the last PMON scan
*/

int fwSystemOverviewFunc_getProjManagersData(string projDp, dyn_string& type, dyn_int& num, dyn_string& opt, dyn_uint& mode, dyn_uint& index, dyn_int& cStat, dyn_int& okStat,
                        dyn_int& pid, dyn_uint& resetMin, dyn_uint& restart, dyn_uint& secKill, dyn_time& startTime, dyn_bool& lastUpdated)
{
  return dpGet(projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_OK_STATE_STR, okStat,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_NUM_STR, num,   
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_TYPE_STR, type,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_INDEX_STR, index,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_MODE_STR, mode, 
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_OPTIONS_STR, opt,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_PID_STR, pid,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_RESETMIN_STR, resetMin,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_RESTART_STR, restart,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_SECKILL_STR, secKill,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_STARTTIME_STR, startTime,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_CURR_STATE_STR, cStat,
               projDp + fwSystemOverviewFunc_MANAGER_STR + fwSystemOverviewFunc_UPDATED_STR, lastUpdated
               ); 
}



/** Gets Access data for polling the project
@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param projDp        Project datapoint name
@param hostName      Host machine name on which the project is running
@param pmonPortNo    Port number of the PMON port
@param userName      User name which has access priviledge to the PMON
@param userPassword  Password for the user
@param sharedDir     Shared directory path for the project folder
*/

int fwSystemOverviewFunc_getProjAccessData(string projDp, string &hostName, unsigned &pmonPortNo, string &userName, string &userPassword, string &sharedDir)
{
  return dpGet(projDp + fwSystemOverviewFunc_HOSTNAME_STR, hostName, 
               projDp + fwSystemOverviewFunc_PMON_PORT_STR, pmonPortNo, 
               projDp + fwSystemOverviewFunc_USER_NAME_STR, userName,
               projDp + fwSystemOverviewFunc_USER_PASSWORD_STR, userPassword,
               projDp + fwSystemOverviewFunc_PROJ_PATH_STR, sharedDir);
}







 
/** Get project datapoint names registered for a system
@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param sysDp      System Datapoint name
@param projDpList Array of project datapoint names
*/

int fwSystemOverviewFunc_getProjDPList(string sysDp, dyn_string& projDpList)
{
  return dpGet(sysDp + fwSystemOverviewFunc_PROJLIST_STR, projDpList);
}
 


// int getParentNode(string dpName, unsigned &parentNode)
// {
//   return dpGet(dpName + PARENT_STR, parentNode);
// }





/** Convert system number to datapoint name
@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param sysNo  System number
*/

string fwSystemOverviewFunc_sysNoToDp(unsigned sysNo)
{
  return fwSystemOverviewFunc_SYSVIEW_NODE + sysNo;
}









/** Get system data
@par Constraints
	None

@par Usage
	Public

@par PVSS managers
	VISION, CTRL

@param sysDp      System datapoint name
@param sysNo      System number
@param sysName    System name
@param parentNo   Parent system number
@param childNodes Children nodes
@param projDpList Project datapoint name list
*/

int fwSystemOverviewFunc_getSysData(string sysDp, unsigned& sysNo, string& sysName, unsigned& parentNo, dyn_uint& childNodes, dyn_string& projDpList)
{
  return dpGet(sysDp + fwSystemOverviewFunc_SYSNO_STR, sysNo, 
               sysDp + fwSystemOverviewFunc_SYSNAME_STR , sysName,
               sysDp + fwSystemOverviewFunc_PARENT_STR , parentNo,
               sysDp + fwSystemOverviewFunc_CHILD_STR , childNodes,
               sysDp + fwSystemOverviewFunc_PROJLIST_STR , projDpList);
}

//@}
